package org.diagbot.common.push.filter.rule;


import org.diagbot.common.push.bean.Rule;
import org.diagbot.common.push.bean.SearchData;
import org.diagbot.common.push.cache.ApplicationCacheUtil;
import org.diagbot.nlp.rule.module.PreResult;
import org.diagbot.nlp.rule.pretreat.*;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class PretreatmentRule {
    public void rule(SearchData searchData) throws java.io.IOException {
        //患者基本信息 性别 年龄
        if (!StringUtils.isEmpty(searchData.getNormal())) {
            add2PreResultList(new PretreatmentNormal(), searchData.getNormal(), "normal", searchData);
        }
        //症状规则 主诉、现病史提取
        if (!StringUtils.isEmpty(searchData.getChief())) {
            searchData.setChief(add2PreResultList(new PretreatmentSymptom(), searchData.getChief(), "symptom", searchData));
        }
        //体征数据 体征、现病史提取
        if (!StringUtils.isEmpty(searchData.getVital())) {
            searchData.setVital(add2PreResultList(new PretreatmentVital(), searchData.getVital(), "vital", searchData));
        }
        //诊断数据  诊断信息(历史)、现病史、既往史(重要疾病史)
        if (searchData.getDiag() != null && searchData.getDiag().size() > 0) {
            List<PreResult> preResults = searchData.getDiag();
            for (PreResult result : preResults) {
                result.setUniqueName("诊断--" + result.getUniqueName());
            }
            searchData.setDiagString(add2PreResultList(searchData.getDiag(), searchData.getDiagString(), "diag", searchData));
            add2PreResultList(new PretreatmentDiag(), searchData.getPasts(), "diag", searchData);
        } else if (!StringUtils.isEmpty(searchData.getDiagString())) {
            searchData.setDiagString(add2PreResultList(new PretreatmentDiag(), searchData.getDiagString(), "diag", searchData));
            add2PreResultList(new PretreatmentDiag(), searchData.getPasts(), "diag", searchData);
        }
        //药品数据  药品信息(历史)、现病史、既往史
        if (searchData.getDrug() != null && searchData.getDrug().size() > 0) {
            List<PreResult> preResults = searchData.getDrug();
            for (PreResult result : preResults) {
                result.setUniqueName("药品--" + result.getUniqueName());
            }
            searchData.setDrugString(add2PreResultList(searchData.getDrug(), searchData.getDrugString(), "drug", searchData));
            add2PreResultList(new PretreatmentDrug(), searchData.getPasts(), "drug", searchData);
        } else if (!StringUtils.isEmpty(searchData.getDrugString())) {
            searchData.setDrugString(add2PreResultList(new PretreatmentDrug(), searchData.getDrugString(), "drug", searchData));
            add2PreResultList(new PretreatmentDrug(), searchData.getPasts(), "drug", searchData);
        }
        //既往史
        if (!StringUtils.isEmpty(searchData.getPasts())) {
            add2PreResultList(new PretreatmentPast(), searchData.getPasts(), "pasts", searchData);
        }
        //手术外伤史 现病史、手术史
        if (!StringUtils.isEmpty(searchData.getPasts())) {
            add2PreResultList(new PretreatmentOperation(), searchData.getOperation(), "operation", searchData);
            add2PreResultList(new PretreatmentOperation(), searchData.getSymptom(), "operation", searchData);
        }
        //过敏 现病史、过敏史
        if (!StringUtils.isEmpty(searchData.getPasts())) {
            add2PreResultList(new PretreatmentAllergy(), searchData.getAllergy(), "allergy", searchData);
            add2PreResultList(new PretreatmentAllergy(), searchData.getSymptom(), "allergy", searchData);
        }
        //外伤   现病史、手术外伤史
        if (!StringUtils.isEmpty(searchData.getPasts())) {
            add2PreResultList(new PretreatmentWound(), searchData.getOperation(), "wound", searchData);
            add2PreResultList(new PretreatmentWound(), searchData.getSymptom(), "wound", searchData);
        }
        //传染病   现病史、传染病史
        if (!StringUtils.isEmpty(searchData.getPasts())) {
            add2PreResultList(new PretreatmentInfectious(), searchData.getInfectious(), "infectious", searchData);
            add2PreResultList(new PretreatmentInfectious(), searchData.getSymptom(), "infectious", searchData);
        }
        //lis结构化信息
        if (searchData.getLis() != null && searchData.getLis().size() > 0) {
            List<PreResult> preResults = searchData.getLis();
            for (PreResult result : preResults) {
                result.setUniqueName("化验--" + result.getUniqueName());
            }
            searchData.setLisString(add2PreResultList(searchData.getLis(), searchData.getLisString(), "lis", searchData));
        } else if (!StringUtils.isEmpty(searchData.getLisString())) {
            searchData.setLisString(add2PreResultList(new PretreatmentLis(), searchData.getLisString(), "lis", searchData));
        }
        //pacs结构化信息
        if (searchData.getPacs() != null && searchData.getPacs().size() > 0) {
            List<PreResult> preResults = searchData.getPacs();
            for (PreResult result : preResults) {
                result.setUniqueName("检查--" + result.getUniqueName());
            }
            searchData.setPacsString(add2PreResultList(searchData.getPacs(), searchData.getPacsString(), "pacs", searchData));
        } else if (!StringUtils.isEmpty(searchData.getPacsString())) {
            searchData.setPacsString(add2PreResultList(new PretreatmentPacs(), searchData.getPacsString(), "pacs", searchData));
        }

        //开具 化验
        if (!StringUtils.isEmpty(searchData.getLisOrder())) {
            add2PreResultList(new PretreatmentMakeList(), searchData.getLisOrder(), "lisOrder", searchData);
        }
        //开具 检查
        if (!StringUtils.isEmpty(searchData.getPacsOrder())) {
            add2PreResultList(new PretreatmentMakeList(), searchData.getPacsOrder(), "pacsOrder", searchData);
        }
        //开具 诊断
        if (!StringUtils.isEmpty(searchData.getDiagOrder())) {
            add2PreResultList(new PretreatmentMakeList(), searchData.getDiagOrder(), "diagOrder", searchData);
        }
        //开具 药品
        if (!StringUtils.isEmpty(searchData.getDrugOrder())) {
            add2PreResultList(new PretreatmentMakeList(), searchData.getDrugOrder(), "drugOrder", searchData);
        }
        //开具 手术
        if (!StringUtils.isEmpty(searchData.getOperationOrder())) {
            add2PreResultList(new PretreatmentMakeList(), searchData.getOperationOrder(), "operationOrder", searchData);
        }

//        Map<String, List<Rule>> ruleMap = searchData.getRules();
//        for (Map.Entry<String, List<Rule>> entry : ruleMap.entrySet()) {
//            for (Rule rule : entry.getValue()) {
//                System.out.println("id:" + rule.getId() + "; pub_name: " + rule.getPub_name());
//            }
//        }
    }

    private String add2PreResultList(Pretreatment pretreatment, String content, String inputType, SearchData searchData) throws java.io.IOException {
        List<PreResult> preResultList = pretreatment.analyze(content);
        return add2PreResultList(preResultList, content, inputType, searchData);
    }

    private String add2PreResultList(List<PreResult> preResultList, String content, String inputType, SearchData searchData) throws java.io.IOException {
        Map<String, List<Rule>> kl_rule_filter_map = ApplicationCacheUtil.getKl_rule_filter_map();
        //符合条件的规则
        Map<String, List<Rule>> accord_rule_map = searchData.getRules();
        List<Rule> accord_rules = null;
        if (preResultList != null) {
            for (PreResult result : preResultList) {
                //规则库中匹配
                if (kl_rule_filter_map.get(result.getUniqueName()) != null) {
                    List<Rule> rules = kl_rule_filter_map.get(result.getUniqueName());
                    if (rules == null) {
                        continue;
                    }
                    for (Rule rule : rules) {
                        boolean isSuit = suitRule(result, rule, content);
                        if (isSuit) {
                            rule.setOriginText(result.getUniqueName() + ":" + result.getValue() + result.getUnits());
                            if (accord_rule_map.get(inputType) == null) {
                                accord_rules = new ArrayList<>();
                            } else {
                                accord_rules = accord_rule_map.get(inputType);
                            }
                            accord_rules.add(rule);
                            accord_rule_map.put(inputType, accord_rules);
                            searchData.setRules(accord_rule_map);

                            content = content + (rule.getRemind() == null ? "" : rule.getSuffixInfo());
                        }
                    }
                }
            }

        }
        return content;
    }

    private boolean suitRule(PreResult result, Rule rule, String content) {
        //标准值最优先匹配
        if (org.apache.commons.lang3.StringUtils.isNotEmpty(rule.getEq_value())) {
            if (compareEqual(result.getValue(), rule.getEq_value())) {
                return true;
            } else {
                return false;
            }
        } else if (org.apache.commons.lang3.StringUtils.isNotEmpty(rule.getMax_value()) && org.apache.commons.lang3.StringUtils.isNotEmpty(rule.getMin_value())) {
            if (compareMin(result.getValue(), rule.getMax_value(), rule.getMax_operator()) //比最大值小
                    && compareMax(result.getValue(), rule.getMin_value(), rule.getMin_operator()) //比最小值大
                    && result.getUnits().equals(rule.getMin_unit())
                    && result.getUnits().equals(rule.getMax_unit())) {
                return true;
            } else {
                return false;
            }
        } else if (org.apache.commons.lang3.StringUtils.isNotEmpty(rule.getMin_value())) {
            if (compareMin(result.getValue(), rule.getMin_value(), rule.getMin_operator())   //比最小值小
                    && result.getUnits().equals(rule.getMin_unit())) {
                return true;
            } else {
                return false;
            }
        } else if(org.apache.commons.lang3.StringUtils.isNotEmpty(rule.getMax_value()))  {
            if (compareMax(result.getValue(), rule.getMax_value(), rule.getMax_operator())   //比最大值大
                    && result.getUnits().equals(rule.getMax_unit())) {
                return true;
            } else {
                return false;
            }
        }
        return false;
    }


    private boolean compareEqual(String c1, String c2) {
        if (!StringUtils.isEmpty(c1) && !StringUtils.isEmpty(c2)
                && c1.equals(c2)) {
            return true;
        }
        return false;
    }

    private boolean compareMin(String c1, String c2, String operator) {
        if (!StringUtils.isEmpty(c1) && !StringUtils.isEmpty(c2) && !StringUtils.isEmpty(operator)) {
            try {
                if (operator.contains("=")) {
                    return Double.valueOf(c1) <= Double.valueOf(c2);
                } else {
                    return Double.valueOf(c1) < Double.valueOf(c2);
                }
            } catch (Exception e) {
            }
        }
        return false;
    }

    private boolean compareMax(String c1, String c2, String operator) {
        if (!StringUtils.isEmpty(c1) && !StringUtils.isEmpty(c2) && !StringUtils.isEmpty(operator)) {
            try {
                if (operator.contains("=")) {
                    return Double.valueOf(c1) >= Double.valueOf(c2);
                } else {
                    return Double.valueOf(c1) > Double.valueOf(c2);
                }
            } catch (Exception e) {
            }
        }
        return false;
    }

}
